@@ -447,7 +447,10 @@ sub draw_text_cell {
 	if ($data -> {bgcolor}) {
 		$data -> {attributes} -> {style} .= "background:$data->{bgcolor};";
 	}
-	
+	if ($data -> {level}) {
+		$data -> {attributes} -> {style} .= "padding-left:" . ($data -> {level} * 12) . "px;";
+	}
+
 	my $attributes = dump_attributes ($data -> {attributes});
 
 	my $txt = '';
